//---------------------------------------------------------------------------------------
//  MKConnectionType.h created by erik on Fri Apr 12 2002
//  @(#)$Id: MKConnectionType.h,v 1.5 2002/12/16 23:26:16 erik Exp $
//
//  Copyright (c) 2002 by Mulle Kybernetik. All rights reserved.
//
//  Permission to use, copy, modify and distribute this software and its documentation
//  is hereby granted, provided that both the copyright notice and this permission
//  notice appear in all copies of the software, derivative works or modified versions,
//  and any portions thereof, and that both notices appear in supporting documentation,
//  and that credit is given to Mulle Kybernetik in all documents and publicity
//  pertaining to direct or indirect use of this code or its derivatives.
//
//  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH MAY HAVE
//  SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF THIS SOFTWARE IN ITS
//  "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY
//  DAMAGES WHATSOEVER RESULTING DIRECTLY OR INDIRECTLY FROM THE USE OF THIS SOFTWARE
//  OR OF ANY DERIVATIVE WORK.
//---------------------------------------------------------------------------------------

#import <InterfaceBuilder/InterfaceBuilder.h>
#import "IBPrivate.h"

@class MKConnectionBrowserCell;


@interface MKConnectionType : NSObject
{
    id <MKIBDocuments>	document;
}


// Applicable connection types for a specific object. Note that the NibOutletConnector
// manager always returns "Outlets" and the NibControlConnector returns "Target".
// The inspector knows about these cases and treats them slightly different. (For
// one their sockets are displayed even if another connection type is displayed.)
// Default implementation returns an array containing one instance of the class.
+ (NSArray *)connectionTypesForObject:(id)anObject;


// Set to the document that contains the current object. Why do we set the document
// but pass in the object and connectors? Consider the document to define a scope
// for the other operations...
- (void)setDocument:(id <MKIBDocuments>)aDocument;
- (id <MKIBDocuments>)document;


// Label for the type to be used in the PopUp button for example.
- (NSString *)name;

// Socket labels for an specific object. These labels must be the same that are 
// used in the sourceLabelForConnector method to allow the inspector to assign
// connectors to sockets.
- (NSArray *)socketLabelsForObject:(id)anObject;

// Socket labels that cannot be used for the given object; because the connection 
// type uses them in another way for example.
- (NSArray *)socketLabelsTakenForObject:(id)anObject;

// This method is called for each socket that this manager is responsible for. It allows
// the  connector class manager to add images, etc. Note that a connector, if present,
// is set as the cell's represented object.
- (void)setupSocketBrowserCell:(MKConnectionBrowserCell *)cell;

// All labels (in order of preference) that describe sockets for anObject which can
// be connected to otherObject. This must be a subset of socketLabelsForObject:
- (NSArray *)preferredSocketLabelsForObject:(id)anObject destination:(id)otherObject;


- (NSArray *)destinationLabelsForObject:(id)anObject path:(NSString *)path;

// Connection type can suggest certain connections should not be made
- (BOOL)canConnectSource:(id)sourceObject socketLabel:(NSString *)label destination:(id)destObject path:(NSString *)path message:(NSString **)messagePtr;

// Creating new connectors. Normally you implement the first method. If you need
// more flexibility override the second; in which case you dont need to worry
// about the first.
- (id <IBConnectors>)connectorForSource:(id)source socketLabel:(NSString *)label destination:(id)destObject path:(NSString *)path;
- (BOOL)addConnectorForSource:(id)source socketLabel:(NSString *)label destination:(id)destObject path:(NSString *)path;

// Check whether an object is the "first responder." You might want to handle this
// as a special case when passed as a destination object...
+ (BOOL)objectIsFirstResponder:(id)anObject;

@end


// You must implement categories that implement the following protocol on the
// connector classes

@protocol MKIBConnectors <IBConnectors>
- (NSString *)sourceLabelForDocument:(id <MKIBDocuments>)document;
- (NSString *)destinationLabelForDocument:(id <MKIBDocuments>)document;
- (NSString *)destinationPath;
- (NSString *)connectionTypeName;
@end


// You can override these methods on NSObject to provide more dynamic information
// about outlets, actions, etc.

@interface NSObject(MKConnectionInfo)
- (NSArray *)outletLabelsForDocument:(id <MKIBDocuments>)document;
- (NSArray *)actionLabelsForDocument:(id <MKIBDocuments>)document;
- (BOOL)followsTargetActionParadigmForDocument:(id <MKIBDocuments>)document;
@end


