//---------------------------------------------------------------------------------------
//  MKNibControlConnectionType.m created by erik on Fri Apr 12 2002
//  @(#)$Id: MKNibControlConnectionType.m,v 1.7 2002/12/16 23:26:16 erik Exp $
//
//  Copyright (c) 2002 by Mulle Kybernetik. All rights reserved.
//
//  Permission to use, copy, modify and distribute this software and its documentation
//  is hereby granted, provided that both the copyright notice and this permission
//  notice appear in all copies of the software, derivative works or modified versions,
//  and any portions thereof, and that both notices appear in supporting documentation,
//  and that credit is given to Mulle Kybernetik in all documents and publicity
//  pertaining to direct or indirect use of this code or its derivatives.
//
//  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH MAY HAVE
//  SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF THIS SOFTWARE IN ITS
//  "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY LIABILITY OF ANY KIND FOR ANY
//  DAMAGES WHATSOEVER RESULTING DIRECTLY OR INDIRECTLY FROM THE USE OF THIS SOFTWARE
//  OR OF ANY DERIVATIVE WORK.
//---------------------------------------------------------------------------------------

#import <InterfaceBuilder/InterfaceBuilder.h>
#import <AppKit/AppKit.h>
#import "MKConnectionBrowserCell.h"
#import "MKNibControlConnectionType.h"


//=======================================================================================
    @implementation MKNibControlConnectionType
//=======================================================================================

static NSString *MKTargetSocketLabel = @"target";
static NSString *MKOutletsConnectionType = @"Outlets";


//---------------------------------------------------------------------------------------

- (NSString *)name
{
    // This is a special case the Inspector knows about.
    return nil;
}


- (NSArray *)socketLabelsForObject:(id)anObject
{
    if([anObject followsTargetActionParadigmForDocument:document] == NO)
        return [NSArray array];
    return [NSArray arrayWithObject:MKTargetSocketLabel];
}


- (NSArray *)preferredSocketLabelsForObject:(id)anObject destination:(id)otherObject
{
    if(([anObject followsTargetActionParadigmForDocument:document] == NO) ||
       ([[anObject actionLabelsForDocument:document] count] == 0))
        return [NSArray array];
    return [NSArray arrayWithObject:MKTargetSocketLabel];
}


//---------------------------------------------------------------------------------------

- (NSArray *)destinationLabelsForObject:(id)anObject path:(NSString *)path
{
    if(path != nil)
        return nil;
    return [anObject actionLabelsForDocument:document];
}


//---------------------------------------------------------------------------------------

- (BOOL)canConnectSource:(id)sourceObject socketLabel:(NSString *)label destination:(id)destObject path:(NSString *)path message:(NSString **)messagePtr;
{
    *messagePtr = nil;
    if((path == nil) || ([path hasSuffix:@":"] == NO))
        return NO;
    return YES;
}


- (id <IBConnectors>)connectorForSource:(id)source socketLabel:(NSString *)label destination:(id)destObject path:(NSString *)path
{
    NSNibOutletConnector	*connector;

    connector = [[[NSNibControlConnector alloc] init] autorelease];
    [connector setSource:source];
    [connector setDestination:[[self class] objectIsFirstResponder:destObject] ? nil : destObject];
    [connector setLabel:path];
    return connector;
}


//=======================================================================================
    @end
//=======================================================================================



//=======================================================================================
    @implementation NSNibControlConnector(MKConnectionType)
//=======================================================================================

- (NSString *)sourceLabelForDocument:(id <MKIBDocuments>)document
{
    return MKTargetSocketLabel;
}


- (NSString *)destinationLabelForDocument:(id <MKIBDocuments>)document
{
    NSString *objectLabel;

    objectLabel = ([self destination] != nil) ? [document classForObject:[self destination]] : @"FirstResponder";
    return [NSString stringWithFormat:@"%@.%@", objectLabel, [self label]];
}


- (NSString *)destinationPath
{
    return [self label];
}


- (NSString *)connectionTypeName
{
    // Why do we say "Outlets" here? Imagine a special connection type is selected and
    // the user selects an object with only a NibControl connector. If we wouldn't say
    // anything here the inspector would keep the selected special connection type
    // rather than switching to "Outlets"
    return MKOutletsConnectionType;
}


//=======================================================================================
    @end
//=======================================================================================
